# 节点管理

CITA 中节点分为共识节点和普通节点，交易由共识节点排序并打包成块，再广播至其他节点，共识完成后即被确认为合法区块。普通节点不参与共识，只同步链上所有的原始数据。

公有链没有节点准入机制，意味着任何节点都可以接入链并同步其全部的数据，在满足一定的条件下都可以参加共识。而 CITA 对于共识节点和普通节点都进行了准入管理。对于身份验证失败的节点，即使该节点能够在网络层与其他 CITA 节点连通，这些 CITA 节点也会拒绝与之建立通讯会话，如此可避免信息泄漏。

## 普通节点管理 (白名单)

目前 CITA 对于节点的准入管理采用白名单的方式。每个节点本地保存节点白名单配置文件，其中记录着允许连接的p2p通信和数据同步的节点，包括其公钥、IP 地址、端口、对应的身份信息等。白名单由管理机构生成并分发，运维人员可对其进行维护和管理，可选择连接若干其他节点同时可配置若干普通节点，使其承担数据分析等工作。

### 相关操作

#### 添加普通节点
编辑节点目录下 `network.toml` ，添加一个 `peers` 条目，`cita-network` 会监控文件的修改自动加载最新的配置。`peers` 条目示例:

``` toml
[[peers]]
id_card = 4
ip = "127.0.0.1"
port = 4004
```

#### 删除普通节点

到对应节点目录下，找到 `network.toml`，删除对应 `peers` 条目即可。

## 共识节点管理

CITA 作为一个面向企业级应用的区块链框架，需要保证监管方能够获得相关的权限对共识节点进行管理，包括增加、删除共识节点等操作。对于共识服务方面，需要对其提供实时读取共识节点列表的接口，而中心化管理的方式无法保证各个节点的共识节点列表的安全性及一致性。CITA 采用合约的方式来实现共识节点的管理，通过区块链上的合约可以保证共识节点的安全性及一致性。

在 CITA 初始化创世块阶段，需要初始化一个管理员地址，其拥有管理员角色，将其写入到每个节点的创世块文件中，共识节点管理合约拥有的一个固定地址也写入其中。创世块内容在初始化以后不允许被修改。区块链正常启动之后，将合约写入到创世块中。链外的操作人员可以通过调用 RPC 接口来实现对共识节点的管理。

对于共识节点的管理，包括添加、删除及获得共识节点。

* 添加操作分为发起和确认，节点先调用发起请求，申请成为共识节点，由管理员(拥有管理员角色的账号)确认才完成了添加操作;
* 删除操作只可由管理员执行;
* 共识服务可获得共识节点列表。

普通节点安装生成后申请成为共识节点，需要进行以下操作：

* 将账号地址提交给管理员;
* 节点发起一个记录其为共识节点的合约，并由管理员完成确认; 
* 和其他节点共同修改本地节点白名单;
* 等待区块数据同步完成后即可参与下一次的共识。

### 共识节点管理合约接口

<table>
  <tr>
    <th>名称</th>
    <th>需要权限</th>
    <th>传入参数</th>
    <th>返回值</th>
    <th>详细描述</th>
  </tr>
  <tr>
    <td>newNode(address)<br/><strong>准备共识节点</strong> </td>
    <td>普通角色</td>
    <td>新增节点地址</td>
    <td>操作是否成功 (bool)</td>
    <td>成功后新节点准备成为共识节点，并将其记录在合约共识节点列表中，同时节点将处于 new 状态</td>
  </tr>
  <tr>
    <td>approveNode(address) <br/> <strong>确认共识节点</strong></td>
    <td>管理员角色</td>
    <td>新增共识节点地址</td>
    <td>操作是否成功 (bool)</td>
    <td>新节点成功准备后，可调用此方法确认节点成为共识节点，同时节点将处于 consensus 状态</td>
  </tr>
  <tr>
    <td>deleteNode(address) <br/> <strong>删除共识节点</strong></td>
    <td>管理员角色</td>
    <td>节点地址</td>
    <td>操作是否成功 (bool)</td>
    <td>成功后节点将从节点列表中删除，同时节点将处于 close 状态</td>
  </tr>
  <tr>
    <td>listNode() <br/> <strong>获取共识节点列表</strong></td>
    <td>普通角色(只读)</td>
    <td>空</td>
    <td>地址列表(address[])</td>
    <td>获取共识节点列表，即状态为 consensus 的节点</td>
  </tr>
  <tr>
    <td>getStatus(address) <br/> <strong>获得节点状态</strong></td>
    <td>普通角色(只读)</td>
    <td>节点地址</td>
    <td>
      节点的状态 (uint8):
      <ul>
        <li>0: close 状态</li>
        <li>1: new 状态</li>
        <li>2: consensus 状态</li>
      </ul>
    </td>
    <td>获取共识节点状态</td>
  </tr>
</table>
